/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ant;

import javax.management.MBeanServerConnection;

import org.apache.tools.ant.BuildException;

/**
 * This class gathers the action on a service assembly
 * 
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public abstract class AbstractServiceAssemblyAntTask extends AbstractJBIAntTask {

    /**
     * Action : shutdown a service assembly
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start a service assembly
     */
    protected static final String START = "start";

    /**
     * Action : stop a service assembly
     */
    protected static final String STOP = "stop";

    /**
     * Action : undeploy a service assembly
     */
    protected static final String UNDEPLOY = "undeploy";

    /**
     * Name of the service assembly
     */
    protected String name;

    /**
     * The action to execute. Must be shutDown, start, stop or undeploy.
     */
    private String action;

    /**
     * Creates a new instance of {@link AbstractServiceAssemblyAntTask}
     * 
     * @param action
     */
    public AbstractServiceAssemblyAntTask(String action) {
        this.action = action;
    }

    /**
     * Perform an action on a service assembly
     * 
     * @param action
     */
    public void doTask() throws Exception {
        if (name == null) {
            throw new BuildException("Missing attribute 'name'");
        }

        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters = new Object[1];
        parameters[0] = name;
        String[] signature = new String[] { "java.lang.String" };
        Object result = connection.invoke(JBIJMXConnectorUtil
                .getDeploymentServiceMBeanName(connection), action, parameters, signature);
        if (result instanceof String) {
            // Extract the service assembly name
            String xml = (String) result;
            if (xml.indexOf("Successfully") > -1) {
                try {
                    log("Service assembly named '" + name + "' has been set to state : " + action);
                } catch (NullPointerException e) {
                }
            } else {
                try {
                    log("Problem during the " + action + " of the service assembly named '" + name
                            + "' :\n" + (String) result);
                } catch (NullPointerException e) {
                    // do nothing
                }
            }
        }
    }

    /**
     * 
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

}
